#define TELNET_DIALOG       // Compiler direktiv - udkommenter hvis ikke skal glde
using System;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Collections.Generic;

namespace SocketServerCmdDisconnectedSession
{
	public class ClientHandler
	{
		private Socket clientSocket;
		private	NetworkStream netStream;
		private	StreamWriter writer;
		private	StreamReader reader;
        private Server server;
        private Session session;
        private EchoService echoservice;

        public ClientHandler(Socket clientSocket, Server server)
		{
			this.clientSocket = clientSocket;
            this.server = server;
		}

		public void RunClient ()
		{
		    netStream = new NetworkStream(clientSocket);
		    writer = new StreamWriter (netStream);
		    reader = new StreamReader (netStream);

            doDialog();

            writer.Close();
		    reader.Close();
		    netStream.Close();
            clientSocket.Shutdown(SocketShutdown.Both);
		    clientSocket.Close();
		}

        public void sendToClient(string text)
        {
            writer.WriteLine(text);
            writer.Flush();
        }
        public string receiveFromClient()
        {
            try
            {
                return reader.ReadLine();
            }
            catch
            {
                return null;
            }
        }

        private void doDialog()
        {
            #if TELNET_DIALOG       // Hvis compilerdirektiv er sat medtages koden
                sendToClient("Server klar"); // opstarts besked til klient
                sendToClient("Indtast SessionId eller 0 for ny"); // opstarts besked til klient
            #endif

            // Skaf session objekt 
            string oldSessionId = receiveFromClient();
            session = server.GetSession(oldSessionId);      // hent evt. session-objekt
            if (session == null)
            {                                               // fantes ikke
                session = server.StartSession();            // start ny session
                session.SessionEchoService = new EchoService();
            }
            echoservice = session.SessionEchoService;       // hent stateful modelobjekt / facade

            #if TELNET_DIALOG       // Hvis compilerdirektiv er sat medtages koden
                sendToClient("SessionId:");
            #endif

            sendToClient(session.SessionId);

            #if DEBUG       // Hvis compilerdirektiv er sat medtages koden
                Console.WriteLine("SessionId: "+session.SessionId+" er nu aktiv");
            #endif

            executeCommand();

            session.ConnectedClientHandler = null;      // afregistrer connected
        }
        public bool executeCommand()  //returner false hvis null eller bye
        {
            #if TELNET_DIALOG       // Hvis compilerdirektiv er sat medtages koden
                sendToClient("Indtast komando"); // besked til klient
            #endif

            string command;
            command = receiveFromClient();
            if (command == null)
                return false;       // ikke mere input
            switch (command.Trim().ToLower())
            {
                case "echo":
                    {   // local scope s varable ikke ses uden for
                        string inputMsg = receiveFromClient();          // hent supplerende parametre fra klient
                        string response = echoservice.Echo(inputMsg);   // udfr metode
                        sendToClient(response);                         // send returvrdi til klient
                    }
                    break;
                case "echoupper":
                    {   // local scope s varable ikke ses uden for
                        string inputMsg = receiveFromClient();          // hent supplerende parametre fra klient
                        string response = echoservice.EchoUpper(inputMsg);// udfr metode
                        sendToClient(response);                         // send returvrdi til klient
                    }
                    break;
                case "echolast":
                    {   // local scope s varable ikke ses uden for
                        string response = echoservice.EchoLast();       // udfr metode
                        sendToClient(response);                         // send returvrdi til klient
                    }
                    break;
                case "bye":
                    server.EndSession(session);     // nedlg session
                    return false;
                default:
                    sendToClient("Ukendt komando");
                    break;
            }
            return true;
        }
	}
}
